'----------------------------Supporting Record Structures----------------

    'Supporting .ZIP file structures
    TYPE LocalFileHeader
        Signature AS LONG
        ExtractVer  AS INTEGER
        GenPurFlag  AS INTEGER
        Method      AS INTEGER
        FTime       AS INTEGER
        FDate       AS INTEGER
        Crc32       AS LONG
        CompSize    AS LONG
        UnCompSize  AS LONG
        FilNamLen   AS INTEGER
        ExtraFldLen AS INTEGER
    END TYPE

    TYPE EndOfCentralDir
        Signature   AS LONG
        DiskNum     AS INTEGER
        DiskWCD     AS INTEGER      'Disk number with start of central dir
        NumThisDisk AS INTEGER      'Number of entries in CD on this disk
        TotalCD     AS INTEGER      'Total number of entires in CD
        CDSize      AS LONG
        CDOffset    AS LONG         'Offset of CD on first disk with CD
        CommentLen  AS INTEGER      'Length of zip file comment
    END TYPE

    'Supporting Lha file structures
    TYPE LhaDirRecFixed
        HeaderSize  AS STRING * 1
        CheckSum    AS STRING * 1
        Method      AS STRING * 5
        CompSize    AS LONG
        UnCompSize  AS LONG
        FTime       AS INTEGER
        FDate       AS INTEGER
        FilAttr     AS INTEGER
        FilNamLen   AS STRING * 1
    END TYPE

'-------------------High-Level Routines-----------------------------------

DECLARE SUB EtCheckPath (Path$)
DECLARE FUNCTION EtCheckDiskSpace%(FileName$,SpaceNeeded&)
DECLARE FUNCTION EtCreatePath% (Path$)
DECLARE FUNCTION EtExpandFileName$ (Name$)
DECLARE FUNCTION EtFindEnd% (InputString$)
DECLARE FUNCTION EtFindSig% (Handle%, Sig$, SigPos&, Dir%)
DECLARE FUNCTION EtFixWildCard$ (Spec$)
DECLARE FUNCTION EtGetStringL$ (Handle%, Size%, Status%)
DECLARE FUNCTION EtGetString0$ (Handle%, Status%)
DECLARE FUNCTION EtMatchFile% (pName$, pSpec$)
DECLARE FUNCTION OkChar% (A$, Intype%, LegalChar$)
DECLARE SUB EtParseCmdLine (Cmd$, Sep$, Params$(), Found%)
DECLARE SUB EtPathName (PathName$, Drive$, Path$, Name$)
DECLARE SUB EtPrintLeftS (Text$, Row%, Col%)
DECLARE SUB EtPrintRightS (Text$, Row%, Col%)
DECLARE SUB EtTextColor ()
DECLARE FUNCTION EtUnZipFile% (InHandle%, OutHandle%, CompSize&, Method%, Flag%, CRC&, PrintOn%)
DECLARE FUNCTION EtZipAttr$ (DosAtt&, Flag%)
DECLARE FUNCTION EtZipFile% (InHandle%, OutHandle%, InFileSize&, CompMethod%, CompLen&, Crc32&)
DECLARE FUNCTION EtZipPercent% (USize&, CSize&)

'------------Low-level Compression/Decompression Routines---------------------

DECLARE FUNCTION EtHuffmanBufSize& ()
DECLARE SUB EtHuffmanInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtHuffmanScan% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)
DECLARE FUNCTION EtHuffman% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtImplodeBufSize& ()
DECLARE SUB EtImplodeInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtImplodeScan% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)
DECLARE FUNCTION EtImplode% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtMashBufSize& ()
DECLARE SUB EtMashParams (BYVAL MaxBits%, BYVAL RleFirst%)
DECLARE SUB EtMashInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtMash% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtScrunchBufSize& ()
DECLARE SUB EtScrunchInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtScrunch% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtShrinkBufSize& ()
DECLARE SUB EtShrinkInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtShrink% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

'----------------------------

DECLARE FUNCTION EtHuffmanUCBufSize& ()
DECLARE SUB EtHuffmanUCInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtHuffmanUC% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtImplodeUCBufSize& ()
DECLARE SUB EtImplodeUCParams (BYVAL ThreeTrees%, BYVAL EightKWindow%)
DECLARE SUB EtImplodeUCInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtImplodeUC% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtMashUCBufSize& ()
DECLARE SUB EtMashUCParams (BYVAL MaxBits%, BYVAL RleFirst%)
DECLARE SUB EtMashUCInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtMashUC% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtScrunchUCBufSize& ()
DECLARE SUB EtScrunchUCInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtScrunchUC% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)

DECLARE FUNCTION EtShrinkUCBufSize& ()
DECLARE SUB EtShrinkUCInit (BYVAL BufSeg%, BYVAL BufOfs%)
DECLARE FUNCTION EtShrinkUC% (BYVAL InBufSeg%, BYVAL InBufOfs%, BYVAL InStart&, InSize&, BYVAL OutBufSeg%, BYVAL OutBufOfs%, BYVAL OutStart&, OutSize&, BYVAL IntVarSeg%, BYVAL IntVarOfs%, BYVAL EndOfInput%)
