DEFINT A-Z
 
  TYPE DBType
    rootfile AS INTEGER
    Drive AS INTEGER
    OEM AS STRING * 8
    BytesPerSector AS INTEGER
    SectorsPerCluster AS INTEGER
    ReservedSectors AS INTEGER
    NumberOfFats AS INTEGER
    RootEntries AS INTEGER
    Sectors AS LONG
    Clusters AS LONG
    MediaType AS STRING * 45
    SectorsPerFAT AS INTEGER
    SectorsPerTrack AS INTEGER
    NumberOfHeads AS INTEGER
    NumberHiddenSectors AS INTEGER
    StartOFRoot AS INTEGER
    StartOFFAT AS LONG
    StartOfData AS INTEGER
    SizeOfDir AS INTEGER
    SizeOfFile AS DOUBLE
  END TYPE

  TYPE RegType
    AX AS INTEGER
    BX AS INTEGER
    CX AS INTEGER
    DX AS INTEGER
    BP AS INTEGER
    SI AS INTEGER
    DI AS INTEGER
    FLAGS AS INTEGER
  END TYPE

  DIM SHARED BPB AS DBType
  DIM SHARED Regs AS RegType

  '---subs needed by DIRALL
  DECLARE SUB Interrupt (h, InRegs AS ANY, OutRegs AS ANY)
  DECLARE SUB PathStrip (filename$, Drive$, Path$)
 
  '---function needed by DIRALL
  DECLARE FUNCTION CurDrive$ ()
  DECLARE FUNCTION NumToBCD$ (num#)
  DECLARE FUNCTION BCDtoNum# (BCD$)
  DECLARE FUNCTION ToNum& (B$)
  DECLARE FUNCTION ToBin$ (num&)
  
  '---DIRALL subs & functions
  DECLARE SUB DisplayDir (Path$, Sector&, Qty%)
  DECLARE SUB FindDir (filename$, StartSector&, SizeOfDir&, Status%)
  DECLARE SUB FindFAT (Cluster&, Sector&, Status%)
  DECLARE SUB GetDOSBoot (Drive$, Status%)
  DECLARE SUB GetStartCluster (filename$, StartSector&, StartCluster&, Status%)
  DECLARE SUB FindCluster (DirName$, ToFind$, Cluster&, size&, SearchAttr$, Status%)
 
  DECLARE FUNCTION DisplayDirEntry$ (Entry$, Mode%)

  '---run in a loop to demo
 
  DO
 
    '---print directions
    DO: LOOP UNTIL INKEY$ = ""
    CLS
    PRINT "Display the directory showing all directory entry information."
    PRINT "You must use a path. Wildcards are not allowed."
    PRINT "For example C:\DOS\ or C:\. You must terminate entry with '\'. "
 
    '---get dir name to display
    LINE INPUT "File name (blank to exit) : "; filename$
    IF LTRIM$(RTRIM$(filename$)) = "" THEN EXIT DO
 
    '---must call to get info needed before calling GetStartCluster
    GetDOSBoot (filename$), Status
    IF Status THEN END

    '---get starting sector for 'filename$'
    FindDir (filename$), StartSector&, SizeOfDir&, Status

    '---setup for DisplayDir
    Sector& = StartSector&                       'root dir of drive
    Qty = BPB.SectorsPerCluster

    '---do the dir
    DisplayDir filename$, Sector&, Qty
   
  LOOP

FUNCTION BCDtoNum# (BCD$)

  'converts up to a 4 character Binary Coded Decimal (BCD)
  ' string into a number. The maximum number is 4,294,967,295

  IF LEN(BCD$) THEN Byte1# = ASC(MID$(BCD$, 1, 1))
  IF LEN(BCD$) > 1 THEN byte2# = ASC(MID$(BCD$, 2, 1))
  IF LEN(BCD$) > 2 THEN byte3# = ASC(MID$(BCD$, 3, 1))
  IF LEN(BCD$) > 3 THEN byte4# = ASC(MID$(BCD$, 4, 1))

  num# = (byte4# * 16777216) + (byte3# * 65536) + (byte2# * 256) + Byte1#

  'assign the function
  BCDtoNum = num#


END FUNCTION

FUNCTION CurDrive$

 Regs.AX = &H1900
 Interrupt &H21, Regs, Regs
 CurDrive$ = CHR$((Regs.AX AND &HFF) + 65) + ":"

END FUNCTION

SUB DisplayDir (Path$, Sector&, Qty)

'this routine displays a directory like DOS, but it also shows erased
'files, starting cluster as well as attribute & size.
 
 
  '---Setup error trap
  Status = 1    'set error flag for error - guilty until proven innocent

  '---parse filename
  Path$ = UCASE$(Path$)
  Drive$ = LEFT$(Path$, 1)
  Drive = ASC(UCASE$(Drive$)) - 65               'change drive to a number

  '---displays "directory" with ALL data
  DO
  
   '---determine the cluster to display
   dCluster& = ((Sector& - BPB.StartOfData) / BPB.SectorsPerCluster) + 2
   IF Sector& < 0 THEN dSector& = 32768 - dSector& ELSE dSector& = Sector&
  
   '---set up a little window to display file info in
   VIEW PRINT 1 TO 25
   CLS
   LOCATE 1, 1
   COLOR 0, 7
   PRINT SPACE$(80);
   LOCATE 1, 1
   PRINT " MAXBASIC directory  "; Path$; "  Sectors";
   PRINT dSector&; "to"; dSector& + 4;
   LOCATE 2, 1
   COLOR 7, 0
   PRINT "File          Status  Attr    Time      Date      Cluster     Sector     Size"
   LOCATE 3, 1
   PRINT STRING$(80, "");
   LOCATE 23, 1
   PRINT STRING$(80, "");
   LOCATE 24, 1
   PRINT "A=ArchiveD=DirectoryH=HiddenR=Read onlyS=SystemV=Volume label";
   LOCATE 25, 1
   PRINT "Files starting with  are erased. Press space for next, escape to quit.";
   VIEW PRINT 4 TO 23
   LOCATE 4, 1
   SizeOfRoot = BPB.StartOfData
 
   '---get directory
   root$ = SPACE$(Qty * BPB.BytesPerSector) + CHR$(0) 'ASCIIZ to recieve FAT
   Regs.AX = Drive                              'Set drive to get
   Regs.BX = SADD(root$)                        'set DS:BX to DOSBOOT$
   Regs.CX = BPB.SectorsPerCluster              'read root worth of sectors
   Regs.DX = Sector&                            'read root starting sector
   Interrupt &H25, Regs, Regs                   'call DOS-absolute read
   Reps = (BPB.BytesPerSector * BPB.SectorsPerCluster) \ 32 'items
  
   q = 1                'reset counter

   VIEW PRINT 4 TO 23   'make a text window
   LOCATE 4, 1          ' "
  
   '---print out the items
   FOR X = 1 TO Reps
    Mode = 0
   
    '---print formatted display entry
    IF Mode = 0 THEN PRINT DisplayDirEntry$(MID$(root$, q, 32), Mode);
    Z = Z + 1
   
    'process key strokes from user
    Key$ = INKEY$
   
    'you could support other keys here, like pageup, pagedown
    'line at a time etc
    SELECT CASE Key$
      CASE CHR$(27)             'escape
	EXIT DO
      'CASE CHR$(0) + CHR$(73)  'page up
      'manipulate X to start displaying from where you want
    END SELECT
  
    IF Z = 19 THEN
      Z = 0
      SLEEP
      LOCATE 4, 1
      FOR Y = 1 TO 19
	PRINT SPACE$(80);
      NEXT
      LOCATE 4, 1
    END IF
  
    q = q + 32
  
   NEXT
 
   VIEW PRINT 1 TO 25
 
   '---get next part of directory for loading
   IF Sector& < 0 THEN Sector& = 65536 + Sector&
   Cluster& = ((Sector& - BPB.StartOfData) / BPB.SectorsPerCluster) + 2
  
   '---find it's starting FAT location
   FindFAT Cluster&, Sector&, Status
   IF Sector& = 0 THEN Status = 1
  
   SLEEP
  
 LOOP UNTIL Status

 BEEP
 VIEW PRINT 1 TO 25
 LOCATE 25, 1
 PRINT SPACE$(80);
 LOCATE 25, 1: PRINT "Press any key to return to main menu.";
 SLEEP
 CLS

END SUB

FUNCTION DisplayDirEntry$ (Entry$, Mode)
 
  'this function converts an entry into a formatted string
  'for displaying or processing

   '---if '' then exit - '' means unformatted or empty
   IF INSTR(Entry$, CHR$(246)) THEN EXIT FUNCTION
  
   'figure file name
   BaseName$ = SPACE$(9)
    LSET BaseName$ = MID$(Entry$, 1, 8)
   ext$ = SPACE$(3)
    LSET ext$ = MID$(Entry$, 9, 3)
   filename$ = BaseName$ + ext$

   'figure file status - erased etc..,
   Stat = ASC(MID$(Entry$, 1, 1))
   SELECT CASE Stat
     CASE 229   ' character
       'erased - E5H
       Status$ = "ERASED"
     CASE 0
       'unused or empty
       Status$ = "EMPTY"
     CASE ELSE
       'normal file
       Status$ = "IN USE"
   END SELECT

   '---this is here for use later on in a file unerase program
   IF Mode = 1 THEN
     IF Status$ <> "ERASED" THEN EXIT FUNCTION
   END IF

   '---figure file attribute(s)
   Atr = ASC(MID$(Entry$, 12, 1))
   Atr$ = SPACE$(6)
    IF (Atr AND 1) <> 0 THEN MID$(Atr$, 4, 1) = "R"
    IF (Atr AND 2) <> 0 THEN MID$(Atr$, 3, 1) = "H"
    IF (Atr AND 4) <> 0 THEN MID$(Atr$, 5, 1) = "S"
    IF (Atr AND 8) <> 0 THEN MID$(Atr$, 6, 1) = "V"
   IF (Atr AND 16) <> 0 THEN MID$(Atr$, 2, 1) = "D"
   IF (Atr AND 32) <> 0 THEN MID$(Atr$, 1, 1) = "A"
  
   'figure time stamp
   Time& = BCDtoNum(MID$(Entry$, 23, 2))
   Hour = (Time& AND &H7FFF) \ 2048
   IF Tim& < 0 THEN Hour = Hour + 16
   Min = ((Time& AND &H7FFF) \ 32) AND &H3F
   Sec = (Time& AND &H1F) * 2
   IF Hour < 10 THEN
      Hour$ = "0" + LTRIM$(STR$(Hour))
   ELSE
      Hour$ = LTRIM$(STR$(Hour))
   END IF
   IF Min < 10 THEN
      Min$ = "0" + LTRIM$(STR$(Min))
   ELSE
      Min$ = LTRIM$(STR$(Min))
   END IF
   IF Sec < 10 THEN
      Sec$ = "0" + LTRIM$(STR$(Sec))
   ELSE
      Sec$ = LTRIM$(STR$(Sec))
   END IF
   TimeStamp$ = Hour$ + ":" + Min$ + ":" + Sec$

   'figure date stamp
   Date& = BCDtoNum(MID$(Entry$, 25, 2))
   Year = (Date& AND &H7FFF) \ 512 + 80
   IF Date& < 0 THEN Year = Year + 64
   Year$ = LTRIM$(STR$(Year))
   Month = ((Date& AND &H7FFF) \ 32) AND &HF
   Day = Date& AND &H1F
   IF Month < 10 THEN
    Month$ = "0" + LTRIM$(STR$(Month))
   ELSE
    Month$ = LTRIM$(STR$(Month))
   END IF
   IF Day < 10 THEN
    Day$ = "0" + LTRIM$(STR$(Day))
   ELSE
    Day$ = LTRIM$(STR$(Day))
   END IF
   DateStamp$ = Month$ + "-" + Day$ + "-" + Year$

   'figure starting cluster
   StartCluster& = BCDtoNum(MID$(Entry$, 27, 2))
   SC$ = SPACE$(5)
   RSET SC$ = STR$(StartCluster&)
  
   'figure starting sector
   StartSector& = (StartCluster& - 2) * BPB.SectorsPerCluster + BPB.StartOfData
   SS$ = SPACE$(7)
   RSET SS$ = STR$(StartSector&)

   'figure file size
   size& = BCDtoNum(MID$(Entry$, 29, 4))
   SZ$ = SPACE$(10)
   RSET SZ$ = STR$(size&)
  
   '---set function
   DirEntry$ = filename$ + "  " + Status$ + "  " + Atr$
   DirEntry$ = DirEntry$ + "  " + TimeStamp$ + "  " + DateStamp$ + "   "
   DirEntry$ = DirEntry$ + SC$ + "    " + SS$ + SZ$
   DisplayDirEntry$ = DirEntry$
   Mode = 0

END FUNCTION

SUB FindCluster (DirName$, ToFind$, Cluster&, size&, SearchAttr$, Status)

' this sub finds ToFind$ (a directory entry) in a given sub-dir (DirName$)
' with SearchAttr$ attributes and returns it's size & cluster.
'
' FADVSHR are the valid options for SearchAttr$ where:
' F = any & all attributes
' A = Archive
' D = sub directory
' V = volume label
' S = system
' H = hidden
' R = read only
'

 '---Setup error trap
 Status = 0    'set flag for we did'nt find it

 '---do it
 Entrys = (size& * BPB.BytesPerSector) \ 32
 X = 1

 FOR Y = 1 TO Entrys
   Entry$ = MID$(DirName$, X, 32)

   '---figure file attribute(s)
   Atr = ASC(MID$(Entry$, 12, 1))
   Atr$ = SPACE$(6)
   IF (Atr AND 1) <> 0 THEN MID$(Atr$, 4, 1) = "R"
   IF (Atr AND 2) <> 0 THEN MID$(Atr$, 3, 1) = "H"
   IF (Atr AND 4) <> 0 THEN MID$(Atr$, 5, 1) = "S"
   IF (Atr AND 8) <> 0 THEN MID$(Atr$, 6, 1) = "V"
   IF (Atr AND 16) <> 0 THEN MID$(Atr$, 2, 1) = "D"
   IF (Atr AND 32) <> 0 THEN MID$(Atr$, 1, 1) = "A"
  
   '---see if this entry has the atr we are looking for
   IF INSTR(Atr$, SearchAttr$) THEN
     WorkDirName$ = MID$(Entry$, 1, 8) + MID$(Entry$, 9, 3)
     IF RTRIM$(WorkDirName$) = RTRIM$(ToFind$) THEN
       Cluster& = BCDtoNum(MID$(MID$(DirName$, X, 32), 27, 2))
       Status = 1 'we found it!
       BPB.SizeOfFile = BCDtoNum(MID$(Entry$, 29, 4))
       size& = X
       EXIT FOR
     END IF
   END IF
   X = X + 32
   NEXT

 IF Status THEN EXIT SUB        'book if we found it
 Status = 0                     'we didnt find in this sector

END SUB

SUB FindDir (filename$, StartSector&, SizeOfDir&, Status)
'
'syntax :
'   inputs  - filename$, StartSector&
'   outputs - StartSector&, SizeOfDir&, Status
'
'   FindDir filename$, StartSector&, SizeOfDir&, Status
'
'function :
'   Searchs through the DOS directory (root -> subdir1 ->...subdirX -> file)
'   to find a filename$. Returns starting sector and the size of this
'   subdirectory (if any). Only used when filename$ has a path in it.
'

  '---Setup error trap
  Status = 1    'set error flag for error - guilty until proven innocent
 
  '---parse filename & drive
  filename$ = UCASE$(filename$)
  Temp$ = filename$
  Drive$ = LEFT$(filename$, 1)                   'parse drive
  Drive = ASC(UCASE$(Drive$)) - 65               'change drive to a number
  filename$ = MID$(filename$, 3, LEN(filename$)) 'reduce filename to path
						 '  without drive
 
  '---get level of sub-dirs
  '   count up sub dir paths in file name & parse them all into an array
  S1 = 1
  X = 1
  IF S1 THEN
    REDIM SD(500)  'StartCluster& up to 500 dirs
    SD(X) = S1
    DO
      X = X + 1
      SD(X) = INSTR(SD(X - 1) + 1, filename$, "\")
      IF SD(X) = 0 THEN
	X = X - 1
	EXIT DO
      END IF
    LOOP
  END IF
  QtySubDirs = X - 1
  REDIM SD$(QtySubDirs)                'build array to StartCluster& subdirs
 
  IF QtySubDirs = 0 THEN                'the file is in a root directory
    StartSector& = BPB.StartOFRoot
    EXIT SUB
  ELSE
    FOR X = 1 TO QtySubDirs
      SD$(X) = MID$(filename$, SD(X) + 1, (SD(X + 1) - 2) - SD(X) + 1)
    NEXT
  END IF

  '---read dir sector & get entry for file
  BootSector = 1                               'determine start of root etc
  StartOFRoot& = BootSector + (BPB.NumberOfFats * BPB.SectorsPerFAT)
  SizeOfRoot = (BPB.RootEntries * 32) \ BPB.BytesPerSector

  '---search for filename path cluster
  size = SizeOfRoot * BPB.BytesPerSector
  SizeOfDir& = SizeOfRoot          'begin in the root and trace the path down
  StartOfDir& = StartOFRoot&       ' to the actual sub-dir location of the path
				   ' of filename$ - so lets start at the root

  FOR X = 1 TO QtySubDirs                                'in sectors
   DirName$ = SPACE$(size) + CHR$(0)        'ASCIIZ to recieve directory
   size = BPB.SectorsPerCluster * BPB.BytesPerSector
   Regs.AX = Drive                              'set drive to get
   Regs.BX = SADD(DirName$)                     'set BX to DirName$
   Regs.CX = SizeOfDir&                         'read 1 sector at a time
   Regs.DX = StartOfDir&                        'read dir starting sector
   Interrupt &H25, Regs, Regs                   'call DOS-absolute read

   ToFind$ = SD$(X)  'assign dir to look for
   SearchAtr$ = "D"
  
   FindCluster DirName$, ToFind$, Sector&, SizeOfDir&, SearchAtr$, Status   'find it...
   IF Status = 1 THEN
     StartCluster& = Sector&
     StartSector& = (Sector& - 2) * BPB.SectorsPerCluster + BPB.StartOfData
     Status = 0 'setup error status to none
     StartOfDir& = StartSector&
   ELSE
     IF StartOfDir& < 0 THEN StartOfDir& = 65536 + StartOfDir&
     Cluster& = ((StartOfDir& - BPB.StartOfData) / BPB.SectorsPerCluster) + 2
     FindFAT Cluster&, StartOfDir&, Status
     IF Status THEN EXIT FOR
   END IF 'status = 1 meaning we found the entry we were looking for
 
   IF X = QtySubDirs THEN EXIT FOR
  
   '---re-init dir string
   DirName$ = ""
   SizeOfDir& = BPB.SectorsPerCluster

  NEXT     'sector to search through

 IF Status THEN EXIT SUB        'something bad happened - so bug out...
 Status = 0                     'no errors
 IF StartSector& > 32768 THEN StartSector& = StartSector& - 65536

END SUB

SUB FindFAT (Cluster&, Sector&, Status)
'
'syntax :
'   inputs  - Cluster&
'   outputs - Cluster&, Sector&, Status
'
'   FindFAT Cluster&, Sector&, Status
'
'function :
'   Converts a cluster number from a dir entry into a FAT offset, then
'   returns the actual cluster and sector number on the disk of the file
'   which owns Cluster&
'
'NOTE :
'   GetDOSBoot MUST have been called BEFORE you can call this sub routine
'   to set up the BPB information which is used here
'

  '---Setup error trap
  Status = 1    'set error flag for error - guilty until proven innocent

  '---Determine FAT field size (12 or 16 bits)
  EnterCluster& = Cluster&
  NumberOfClusters& = BPB.SectorsPerCluster * BPB.Sectors&
  IF NumberOfClusters& <= 0 THEN
    'error
    EXIT SUB
  ELSEIF NumberOfClusters& > 4087 THEN
    'hard disk > 10 MB use 16 bit FATS
    FATFieldSize = 16
  ELSEIF NumberOfClusters& <= 4087 THEN
    'floppies use 12 bit FATS
    FATFieldSize = 12
  ELSE
    'error
    EXIT SUB
  END IF

  '---calculate FAT offset of cluster
  IF Cluster& <= BPB.StartOfData AND BPB.rootfile THEN
     'this must be a big root dir so pop the cluster to the next one
     Cluster& = Cluster& + 1
     Sector& = Cluster&
     Status = 0
     Sector& = (Cluster& - 2) * BPB.SectorsPerCluster + BPB.StartOfData
     SizeOfRoot = BPB.StartOfData
     IF Sector& > SizeOfRoot THEN Status = 1
     EXIT SUB
  END IF

 '---fixup for negative number
 IF Cluster& < 0 THEN Cluster& = 65536 + Cluster&

 '---determine actual cluster number
 IF FATFieldSize = 16 THEN
   'must be >10M fixed disk
   WorkCluster& = Cluster& * 2
   FATFieldSize = 2
 ELSE
   'must be 12 which is a floppy or <10M fixed
   WorkCluster& = FIX(Cluster& * 1.5)
   FATFieldSize = 2             'will be reading 2 bytes of data
   WholeNum = 0
   IF WorkCluster& = Cluster& * 1.5 THEN WholeNum = 1
 END IF

 '---check for bogus cluster number
 IF WorkCluster& > 65536 THEN EXIT SUB 'error out

 '---load & process the FAT
 FatSize = (BPB.BytesPerSector)
 SectorToLoad = FIX((WorkCluster& / BPB.BytesPerSector)) + 1
 WorkPointer& = WorkCluster& - (SectorToLoad * BPB.BytesPerSector)
 IF WorkPointer& < 0 THEN WorkPointer& = BPB.BytesPerSector + WorkPointer&

 FAT$ = SPACE$(FatSize) + CHR$(0)   'ASCIIZ to recieve FAT
 Regs.AX = BPB.Drive                          'Set drive to read from
 Regs.BX = SADD(FAT$)                         'set DS:BX to DOSBOOT$
 Regs.CX = 1                                  'read FAT sector
 Regs.DX = SectorToLoad                       'read FAT starting sector
 Interrupt &H25, Regs, Regs                   'call DOS-absolute read
 
 'figure starting cluster
 Work$ = MID$(FAT$, WorkPointer& + 1, FATFieldSize)   'get FAT entry and convert
 Cluster& = BCDtoNum(Work$)

 IF NumberOfClusters& > 4087 THEN
    '---16 bit hard disk FAT
    IF Cluster& >= 65535 THEN
      Status = 1
      EXIT SUB
    ELSE
      Status = 0  'end of file
    END IF
 ELSEIF NumberOfClusters& <= 4087 THEN
    '---12 bit floppy FAT
    n$ = ToBin(Cluster&)
    IF WholeNum = 1 THEN                '12 bit FAT with even product
      Cluster& = ToNum(RIGHT$(n$, 12))
    ELSE                                '12 bit FAT with odd product
      Cluster& = ToNum(LEFT$(n$, 12))
    END IF
   
    '---check for end of FAT chain
    IF Cluster& >= &HFF8 THEN
      Status = 1
    ELSE
      Status = 0
    END IF
 END IF
  
 '---set sector & exit
 Sector& = (Cluster& - 2) * BPB.SectorsPerCluster + BPB.StartOfData
 IF Sector& > 32768 THEN Sector& = Sector& - 65536

END SUB

SUB GetDOSBoot (Drive$, Status)

'This sub reads the DOS BIOS Paramater Block (BPB) and determines
'the basic disk configuration for sectors etc..,

'-Setup error trap-------------------------------------------------------

  Status = 1    'set error flag for error - guilty until proven innocent

'Convert drive into a number for DOS-------------------------------------
    Drive$ = LEFT$(Drive$, 1)            'drive$ should be A,B,C etc.,
    BPB.Drive = ASC(UCASE$(Drive$)) - 65     'change drive to a number

'Set disk sector size----------------------------------------------------

   BytesInSector = 512                  'BytesInSector in IBM world

'Read DOS Base Pointer (BPB)---------------------------------------------

   'most interupts added in DOS 3.0 and up, though not all, use a null
   ' terminated string for these calls. We do that in BASIC by appending
   ' CHR$(0) to the end of the string, as shown below. This is referd to
   ' to as ASCIIZ
   'DOS interrupt &H25 is the DOS absolute read function. It will read
   ' as many bytes as the string passed to it is long. So we are going
   ' to read the whole boot sector - which is BytesInSector long - from
   ' above.
   'We call it with the following paramaters
   ' AX = drive to read
   ' BX = pointer to ASCIIZ string to recieve disk information
   ' CX = quantity of sectors to read
   ' DX = absolute disk sector to read

   DOSBOOT$ = SPACE$(BytesInSector) + CHR$(0)   'ASCIIZ to recieve BPB data

   Regs.AX = BPB.Drive                          'Set drive to get

   Regs.BX = SADD(DOSBOOT$)                     'set DS:BX to DOSBOOT$
     'using SADD points to the memory location
     ' of the string. Here we are not actually working
     ' on the string itself - we are working on it indirectly
     ' using what is refered to as "pointers"

   Regs.CX = 1                                  'read '1' sector
   Regs.DX = 0                                  'read sector '0' boot sector
   Interrupt &H25, Regs, Regs                   'call DOS-absolute read

   IF DOSBOOT$ = SPACE$(BytesInSector) + CHR$(0) THEN EXIT SUB
     'the bove line is true if there was an error - so if there was
     ' an error lets boogy...note that Status was preset to indicate
     ' an error - so the calling program knows that this call was
     ' unscucessful.

   'DOSBOOT$ now StartCluster&s the DOS boot record

'Parse BPB info------------------------------------------------------------
   'Ok, if we got here, then we had a successful read. Lets begin to
   ' parse out the boot record information contained in the string DOSBOOT$
   ' which holds a copy of the boot sector. We will use the MID$ function of
   ' BASIC to retrieve sub strings out of the Boot record strings DOSBOOT$.
   ' As follows :
   ' SubString = MID$(DOSBOOT$, Offset into boot sector, Bytes to fetch)

   'get oem info...
   BPB.OEM = MID$(DOSBOOT$, 4, 8)

   'figure bytes/sector. In most DOS applications numbers are converted into
   ' characters, then written to disk. This is called Binart coded decimal
   ' For example dBASE does this also. To convert such a character string
   ' into a number, first read the string then break it into sub strings of
   ' one character each. Then, depending on which character is which, convert
   ' into a number as follows, leftmost character first times decreasing
   ' powers of 16.

   Work$ = MID$(DOSBOOT$, 12, 2)        'get sub string
   Byte1& = ASC(LEFT$(Work$, 1))        'convert each character into ASCII
   byte2& = ASC(RIGHT$(Work$, 1))       '  "       "     "        "   "

   BPB.BytesPerSector = (byte2& * 256) + Byte1&  'convert into number

   'figure sectors/fat cell (allocation unit)
   Work$ = MID$(DOSBOOT$, 14, 1)
   Byte1& = ASC(LEFT$(Work$, 1))
   BPB.SectorsPerCluster = Byte1&

   'figure reserved sectors
   Work$ = MID$(DOSBOOT$, 15, 2)
   Byte1& = ASC(LEFT$(Work$, 1))
   byte2& = ASC(RIGHT$(Work$, 1))
   BPB.ReservedSectors = (byte2& * 256) + Byte1& - 1
     'the above subtraction of 1 from the last line is an offset for a DOS
     ' oddity because BPB.ReservedSectors also shows the boot sector itself

   'figure number of fats
   Work$ = MID$(DOSBOOT$, 17, 1)
   Byte1& = ASC(LEFT$(Work$, 1))
   BPB.NumberOfFats = Byte1&

   'figure maximum root directory entries
   Work$ = MID$(DOSBOOT$, 18, 2)
   Byte1& = ASC(LEFT$(Work$, 1))
   byte2& = ASC(RIGHT$(Work$, 1))
   BPB.RootEntries = (byte2& * 256) + Byte1&

   'figure number of sectors
   Work$ = MID$(DOSBOOT$, 20, 2)
   Byte1& = ASC(LEFT$(Work$, 1))
   byte2& = ASC(RIGHT$(Work$, 1))
   BPB.Sectors& = (byte2& * 256) + Byte1&

   'figure media type where it is stored in HEX format, so first we
   ' read out its bytes
   Work$ = MID$(DOSBOOT$, 22, 1)

   'then we convert is ASC value into HEX
   MediaType$ = HEX$(ASC(LEFT$(Work$, 1)))

   Base$ = "[" + HEX$(ASC(LEFT$(Work$, 1))) + "H]"
   SELECT CASE MediaType$
     CASE "F8"
      MediaDesc$ = " hard disk drive"
     CASE ELSE
      MediaDesc$ = " floppy disk drive"
   END SELECT
   BPB.MediaType = Base$ + MediaDesc$

   'figure number of sectors/fat
   Work$ = MID$(DOSBOOT$, 23, 2)
   Byte1& = ASC(LEFT$(Work$, 1))
   byte2& = ASC(RIGHT$(Work$, 1))
   BPB.SectorsPerFAT = (byte2& * 256) + Byte1&

   'figure clusters
   BPB.Clusters = BPB.Sectors& \ BPB.SectorsPerCluster

   'figure sectors/disk track
   Work$ = MID$(DOSBOOT$, 25, 2)
   Byte1& = ASC(LEFT$(Work$, 1))
   byte2& = ASC(RIGHT$(Work$, 1))
   BPB.SectorsPerTrack = (byte2& * 256) + Byte1&

   'figure number of disk heads
   Work$ = MID$(DOSBOOT$, 27, 2)
   Byte1& = ASC(LEFT$(Work$, 1))
   byte2& = ASC(RIGHT$(Work$, 1))
   BPB.NumberOfHeads = (byte2& * 256) + Byte1&

   'figure number (if any) of hidden sectors
   Work$ = MID$(DOSBOOT$, 29, 2)
   Byte1& = ASC(LEFT$(Work$, 1))
   byte2& = ASC(RIGHT$(Work$, 1))
   BPB.NumberHiddenSectors = (byte2& * 256) + Byte1&

   '------------------get FAT start address-------------------
   'the FAT starts right after (boot_sector + reserved_sectors)
   'so start of FAT is...0 + BPB.ReservedSectors + 1

   BPB.StartOFFAT = BPB.ReservedSectors + 1     '+1 to offset -1 from DOS
						'reserved sectors count
   '------------------get root start address-------------------
   'After the boot sector and the FAT comes the root directory
   'Add 1 to account for the boot sector itself

   BPB.StartOFRoot = 1 + (BPB.NumberOfFats * BPB.SectorsPerFAT)

   '------------------get disk data start address-------------------
   'After the boot sector, the FAT and the root directory, starts
   'the actual disk space that we can use...

   BootSector = 1
   StartOfData = BootSector + (BPB.NumberOfFats * BPB.SectorsPerFAT)
   StartOfData = StartOfData + ((BPB.RootEntries * 32) \ BPB.BytesPerSector)

   BPB.StartOfData = StartOfData

   '------------------end of routine -------------------
   'if we got here then we had no errors so set error flag Status
   ' to show no errors. The Type array BPB now holds all the DOS
   ' boot sector information for display
   '
   Status = 0   'we made with no errors

END SUB

FUNCTION NumToBCD$ (num#)
 
   'Given four characters, the highest number possible is
   '4,294,967,295 - more than 4 BILLION! If you need more, then
   ' add a Byte5# etc.
 
   IF num# > 4294967295# * 256# THEN
     NumToBCD = "overflow"
     EXIT FUNCTION
   END IF
 
   BCDNum# = num#

   byte4# = INT(BCDNum# / 16777216)
     BCDNum# = BCDNum# - byte4# * 16777216
 
   byte3# = INT(BCDNum# / 65536)
     BCDNum# = BCDNum# - byte3# * 65536
 
   byte2# = INT(BCDNum# / 256)
     BCDNum# = BCDNum# - byte2# * 256
 
   Byte1# = BCDNum#
 
   'set size of field
   IF byte4# > 0 THEN size = 1
   IF byte3# > 0 THEN size = size + 1
   IF byte2# > 0 THEN size = size + 1
   IF Byte1# > 0 THEN size = size + 1
 
   'setup a buffer to use
   type$ = SPACE$(size)

   'convert to ascii
   IF Byte1# > 0 THEN MID$(type$, 1, 1) = CHR$(Byte1#)
   IF byte2# > 0 THEN MID$(type$, 2, 1) = CHR$(byte2#)
   IF byte3# > 0 THEN MID$(type$, 3, 1) = CHR$(byte3#)
   IF byte4# > 0 THEN MID$(type$, 4, 1) = CHR$(byte4#)

   'assign function
   NumToBCD = type$
 
END FUNCTION

SUB PathStrip (filename$, Drive$, Path$)

    count = LEN(filename$)

    FOR X = 1 TO count
      IF MID$(filename$, X, 1) = "\" THEN lastdir = X
    NEXT
  
    IF lastdir = 0 THEN
      Path$ = ""
    ELSE
      Path$ = LEFT$(filename$, lastdir)
    END IF
  
    IF INSTR(filename$, ":") THEN
      Drive$ = MID$(filename$, INSTR(filename$, ":") - 1, 3)
      filename$ = RIGHT$(filename$, LEN(filename$) - lastdir)
    ELSE
      Drive$ = CurDrive$
    END IF
 

END SUB

FUNCTION ToBin$ (NumToChange&)

  'This function changes a number less than 65535 into a 16 digit binary
  ' number. For example X$ = ToBin$(8) > '0000000000001000'

  num& = NumToChange&   'So we don't muck-up the value passed to us. This
			' is just polite programming (for your own POM!)

  size = 16


  'Maybe later I will make it work on 32 bit numbers...
			' but for now 16 bits if fine

  n$ = STRING$(16, "0") 'Set our prospective number to all '0s'

  ' -----------------------------------------------------------

  FOR X = size TO 1 STEP -1
 
    'This code below was substituted for a MUCH simpler
    ' N& = 2 ^ X to save time. It bought about .5 second
    ' over using the BASIC ^ arithmetic command!
  
    SELECT CASE X
      CASE 16
	n& = 65536
      CASE 15
	n& = 32768
      CASE 14
	n& = 16384
      CASE 13
	n& = 8192
      CASE 12
	n& = 4096
      CASE 11
	n& = 2048
      CASE 10
	n& = 1024
      CASE 9
	n& = 512
      CASE 8
	n& = 256
      CASE 7
	n& = 128
      CASE 6
	n& = 64
      CASE 5
	n& = 32
      CASE 4
	n& = 16
      CASE 3
	n& = 8
      CASE 2
	n& = 4
      CASE 1
	n& = 2
    END SELECT
  
    N2& = (n& \ 2)
  
    IF num& <= n& AND num& >= N2& THEN
      MID$(n$, 17 - X, 1) = "1"        '17 - X 'cause X is from 16 to 1
				       ' and we want to start at 16...1
				       ' so 17-16=1; 17-15=2; etc.,
      num& = num& - N2&
    END IF
 
  NEXT
 
  ToBin$ = n$

END FUNCTION

FUNCTION ToNum& (B$)

  'This function takes a string in the form of '001010' or any other
  ' binary number and converts it into a long integer. The length of
  ' B$ determines where the translation begins. This is good for doing
  ' MID$ or LEFT$ or RIGHT$ extractions from strings. For example the
  ' FAT of a floppy drive uses a 12 bit number - so you read to bytes
  ' and the take the left or right -most 12 bits & convert it into a
  ' number.
  '
  ' For example:
  '         FloppyFAT = ToNum&(ToBin$(MID$(FAT$,OffSet,2))
  '

  size = LEN(B$)                'establish the length of the string & hence
				' the start count for the loop below

  FOR X = size TO 1 STEP -1     'counting backwards from Size...

    IF MID$(B$, X, 1) = "1" THEN num& = num& + (2 ^ (size - X))

    'if the value of the bit is 1 then our number = 2 to the power
    ' of the position of this bit - which is bit Size-X.

  NEXT

  ToNum& = num&                 'assign it boogy...

END FUNCTION

